(* $Novell: bind.c,v 1.8 2003/05/12 12:14:11 $ */
/**************************************************************************
*  Novell Software Developer Kit
*
*  Copyright (C) 2002-2003 Novell, Inc. All Rights Reserved.
*
*  THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
*  USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE LICENSE AGREEMENT
*  ACCOMPANYING THE SOFTWARE DEVELOPER KIT (SDK) THAT CONTAINS THIS WORK.
*  PURSUANT TO THE SDK LICENSE AGREEMENT, NOVELL HEREBY GRANTS TO DEVELOPER A
*  ROYALTY-FREE, NON-EXCLUSIVE LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS
*  PRODUCT. NOVELL GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
*  DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF DEVELOPER'S
*  PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO DEVELOPER OR DEVELOPER'S
*  CUSTOMERS WITH RESPECT TO THIS CODE.
*
***************************************************************************
    bind.c
***************************************************************************
   Description: The bind.c sample demonstrates both an anonymous and simple
                LDAP v3 bind to an LDAP server.
***************************************************************************)
program bind;
{$APPTYPE CONSOLE}

uses sysutils, ldap;

//static char usage[] =
//"\nUsage:   bind <host name> <port number> <login dn> <password>"
//"\nExample: bind Acme.com 389 cn=admin,o=Acme secret\n";

{$IFDEF VER210}
var
    timeout : timeval = (tv_sec:10;tv_usec:0); //* 10 second connection timeout */
{$ELSE}
const
    timeout : timeval = (tv_sec:10;tv_usec:0); //* 10 second connection timeout */
{$ENDIF}
var
    rc, ldapPort, version: int32;
    ldapHost  : AnsiString;
    loginDN   : AnsiString;
    password  : AnsiString;

    ld        :  hLDAP;
BEGIN
    if (paramcount()<>4) then begin
      writeln('Usage:   bind <host name> <port number> <login dn> <password>');
      writeln('    Example: bind Acme.com 389 cn=admin,o=Acme secret');
      halt(1);
    end;

    ldapHost     := paramstr(1);
    ldapPort     := StrToInt(paramstr(2));
    loginDN      := paramstr(3);
    password     := paramstr(4);

    ///*
    // * Set LDAP version to 3.
    // * Using a NULL session handle sets the global options.
    // * All subsequent LDAP sessions created in this process will be version 3.
    // * Also set the connection timeout.
    // */
    version := LDAP_VERSION3;
    writeln('ldap_set_option');
    ldap_set_option(NIL, LDAP_OPT_PROTOCOL_VERSION, version);
    ldap_set_option(NIL, LDAP_OPT_NETWORK_TIMEOUT, timeOut);

    ///*
    // * Initialize the LDAP session
    // */
    writeln('ldap_init');
    ld := ldap_init(pchar8(ldapHost),ldapPort);

    if (ld = nil) then begin
      writeln('LDAP session initialization failed');
      halt(1);
    end;
    writeln('LDAP session initialized');

    ///*
    // * Bind to the server anonymously
    // */
    writeln('ldap_simple_bind_s');
    rc := ldap_simple_bind_s(ld, nil, nil);
    if (rc <> LDAP_SUCCESS) then begin
      writeln('ldap_simple_bind_s: ',ldap_err2string(rc));
      ldap_unbind_s(ld);
      halt(1);
    end;
    writeln('Anonymous bind successful');

    writeln('ldap_unbind_s');
    ldap_unbind_s(ld);

    ///*
    // * Initialize a new LDAP session
    // */
    writeln('ldap_init');
    ld := ldap_init(pchar8(ldapHost), ldapPort);
    if (ld = nil) then begin
      writeln('LDAP session initialization failed');
      halt(1);
    end;
    writeln('New LDAP session initialized');

    ///*
    // * Simple bind to the server
    // */
    writeln('ldap_simple_bind_s');
    rc := ldap_simple_bind_s(ld, pchar8(loginDN), pchar8(password));
    if (rc <> LDAP_SUCCESS) then begin
      writeln('ldap_simple_bind_s: ', ldap_err2string(rc));
      ldap_unbind_s(ld);
      halt(1);
    end;
    writeln('Bind and authentication to the server successful');

    ldap_unbind_s(ld);
end.
