(* $Novell: gssbind.c,v 1.8 2003/05/12 12:14:11 $ */
/**************************************************************************
*  Novell Software Developer Kit
*
*  Copyright (C) 2002-2004 Novell, Inc. All Rights Reserved.
*
*  THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
*  USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE LICENSE AGREEMENT
*  ACCOMPANYING THE SOFTWARE DEVELOPER KIT (SDK) THAT CONTAINS THIS WORK.
*  PURSUANT TO THE SDK LICENSE AGREEMENT, NOVELL HEREBY GRANTS TO DEVELOPER A
*  ROYALTY-FREE, NON-EXCLUSIVE LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS
*  PRODUCT. NOVELL GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
*  DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF DEVELOPER'S
*  PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO DEVELOPER OR DEVELOPER'S
*  CUSTOMERS WITH RESPECT TO THIS CODE.
*
***************************************************************************
    gssbind.c
***************************************************************************
   Description: 
   The gssbind.c sample demonstrates LDAP v3 SASL bind to an LDAP server 
   using GSSAPI authentication mechanism.

   GSSAPI internally uses kerberos TGT to authenticate. So you should 
   have got kerberos TGT on the client machine before doing GSSAPI bind. 
   You can use kinit to get TGT from KDC. 

   eDirectory 8.8 or newer is required.
***************************************************************************)
program gssbind;
{$APPTYPE CONSOLE}

uses sysutils, ldap_gss, ldap, lber;

//static char usage[] = "\n\tUsage:   gssbind <host name> <port number>"
//                      "\n\tExample: gssbind Acme.com 389 \n";

function isGSSAPISupported(ld: hLDAP): integer;
label
    999;
var
    rc          : integer;
    attribute   : pchar8;
    values      : ppchar8;
    val         : ppchar8;
    ber         : BerElement;
    searchResult: LDAPMessage;
    entry       : LDAPMessage;
    attribs     : array[0..1] of pchar8;
begin
    attribute    := nil;
    values       := nil;
    ber          := nil;
    searchResult := nil;
    entry        := nil;
    attribs[0]   := 'supportedsaslmechanisms';
    attribs[1]   := nil;

    ///* Reading the root DSE */
    rc := ldap_search_ext_s(ld, '' , LDAP_SCOPE_BASE, '(objectClass=*)',
                           ADDR(attribs), 0,nil, nil, nil, LDAP_NO_LIMIT, searchResult);

    if (rc<>LDAP_SUCCESS) then begin
        writeln('ldap_search_ext_s: ', ldap_err2string( rc ));
        ldap_msgfree(searchResult);
        ldap_unbind_s(ld);
        halt(0);
    end;

    entry := ldap_first_entry(ld,searchResult);

    attribute := ldap_first_attribute(ld,entry,ber);
    while (attribute <> nil) do begin
//    for (attribute = ldap_first_attribute( ld, entry, &ber );
//           attribute != NULL;
//           attribute = ldap_next_attribute( ld, entry, ber ))

        ///* Get values and print.  Assumes all values are strings. */
        values := ldap_get_values(ld,entry,attribute);
        if (values <> nil) then begin
           val := values;
           while (val^<>nil) do begin
              if (strcomp(val^,'GSSAPI') = 0) then begin
                 rc := 1;
                 goto 999;
              end;
              INC(val);
            end;
            ldap_value_free(values);
          end;
        ldap_memfree(attribute);
        attribute := ldap_next_attribute(ld,entry,ber);
    end;
    ber_free(ber, 0);

999:
    ldap_msgfree(searchResult);
    result := rc;
end; //-------------------------------------------------------------------------

{$IFDEF VER210}
var
    timeout : timeval = (tv_sec:10;tv_usec:0); //* 10 second connection timeout */
{$ELSE}
const
    timeout : timeval = (tv_sec:10;tv_usec:0); //* 10 second connection timeout */
{$ENDIF}
var
    rc, ldapPort, version: int32;
    ldapHost  : AnsiString;

    ld        :  hLDAP;

    gerr_code : gss_err_code;
BEGIN
    if (paramcount()<>2) then begin
      writeln('Usage:   gssbind <host name> <port number>');
      writeln('    Example: gssbind Acme.com 389');
      halt(1);
    end;

    ldapHost     := paramstr(1);
    ldapPort     := StrToInt(paramstr(2));

    ///*
    // * Set LDAP version to 3.
    // * Using a NULL session handle sets the global options.
    // * All subsequent LDAP sessions created in this process will be version 3.
    // * Also set the connection timeout.
    // */
    version := LDAP_VERSION3;
    writeln('ldap_set_option');
    ldap_set_option(NIL, LDAP_OPT_PROTOCOL_VERSION, version);
    ldap_set_option(NIL, LDAP_OPT_NETWORK_TIMEOUT, timeOut);

    ///*
    // * Initialize the LDAP session
    // */
    writeln('ldap_init');
    ld := ldap_init(pchar8(ldapHost),ldapPort);

    if (ld = nil) then begin
      writeln('LDAP session initialization failed');
      halt(1);
    end;
    writeln('LDAP session initialized');

    if (isGSSAPISupported(ld)=1) then begin


         writeln('ldap_gssbind');
         ///* GSS bind to the server */
         rc := ldap_gssbind(ld,
                            pchar8(ldapHost),
                            pchar8('GSSAPI'), // Note: "GSSAPI" provided to ldap_gssbind is casesensitive
                            NIL,
                            NIL,
                            gerr_code); 

         if (rc<>LDAP_SUCCESS) then begin
             if(rc=_LDAP_GSS_ERROR) then begin
               writeln('tldap_simple_bind_s: ', 'ldap_gss_error(gerr_code)');
             end else begin
               writeln('tldap_simple_bind_s: ', ldap_err2string(rc));
             end;
             ldap_unbind_s(ld);
             writeln('major = ','gerr_code.maj_stat',', minor = ','gerr_code.min_stat');
             halt(1);
         end;

         writeln('Bind and authentication to the server successful');
    end else begin
        writeln('No GSS support from the LDAP server.');
    end;

    ldap_unbind_s(ld);
end.


