(* $Novell: ldap_ssl.h,v 1.32 2003/05/13 13:52:32 $
 *****************************************************************************
 *  Novell Software Developer Kit
 *
 *  Copyright (C) 2002-2003 Novell, Inc. All Rights Reserved.
 *
 *  THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
 *  USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE LICENSE AGREEMENT
 *  ACCOMPANYING THE SOFTWARE DEVELOPER KIT (SDK) THAT CONTAINS THIS WORK.
 *  PURSUANT TO THE SDK LICENSE AGREEMENT, NOVELL HEREBY GRANTS TO DEVELOPER A
 *  ROYALTY-FREE, NON-EXCLUSIVE LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS
 *  PRODUCT. NOVELL GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
 *  DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF DEVELOPER'S
 *  PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO DEVELOPER OR DEVELOPER'S
 *  CUSTOMERS WITH RESPECT TO THIS CODE.
 *****************************************************************************)

unit ldap_ssl;

interface

uses ldap;

type    char8 = ansichar;
        pchar8 = pansichar;

const
        LDAPSSL_SUCCESS        =     0;
        LDAPSSL_ERROR          =    -1;

        LDAPSSL_VERIFY_NONE    =   $00;
        LDAPSSL_VERIFY_SERVER  =   $01;

        LDAPSSL_FILETYPE_B64   =     1;
        LDAPSSL_FILETYPE_DER   =     2;
        LDAPSSL_BUFFTYPE_B64   =     3;
        LDAPSSL_BUFFTYPE_DER   =     4;
        LDAPSSL_FILETYPE_P12   =     5;
        LDAPSSL_BUFFTYPE_P12   =     6;

        LDAPSSL_CERT_FILETYPE_B64  =   LDAPSSL_FILETYPE_B64;
        LDAPSSL_CERT_FILETYPE_DER  =   LDAPSSL_FILETYPE_DER;
        LDAPSSL_CERT_BUFFTYPE_B64  =   LDAPSSL_BUFFTYPE_B64;
        LDAPSSL_CERT_BUFFTYPE_DER  =   LDAPSSL_BUFFTYPE_DER;

        LDAPSSL_CERT_ATTR_ISSUER            =   1;
        LDAPSSL_CERT_ATTR_SUBJECT           =   2;
        LDAPSSL_CERT_ATTR_VALIDITY_PERIOD   =   3;

        LDAPSSL_CERT_GET_STATUS             = 100;

//*
//* Certificate Status
//*/
//* unable to get issuer certificate */
        UNABLE_TO_GET_ISSUER_CERT          =   2;
//* unable to decode issuer public key */
        UNABLE_TO_DECODE_ISSUER_PUBLIC_KEY =   6;
//* certificate signature failure */
        CERT_SIGNATURE_FAILURE             =   7;
//* certificate is not yet valid */
        CERT_NOT_YET_VALID                 =   9;
//* CRL is not yet valid */
        CERT_HAS_EXPIRED                   =  10;
//* format error in certificate's notBefore field */
        ERROR_IN_CERT_NOT_BEFORE_FIELD     =  13;
//* format error in certificate's notAfter field */
        ERROR_IN_CERT_NOT_AFTER_FIELD      =  14;
//* self signed certificate */
        DEPTH_ZERO_SELF_SIGNED_CERT        =  18;
//* self signed certificate in certificate chain */
        SELF_SIGNED_CERT_IN_CHAIN          =  19;
//* unable to get local issuer certificate */
        UNABLE_TO_GET_ISSUER_CERT_LOCALLY  =  20;
//* unable to verify the first certificate */
        UNABLE_TO_VERIFY_LEAF_SIGNATURE    =  21;
//* invalid CA certificate */
        INVALID_CA                         =  24;
//* path length constraint exceeded */
        PATH_LENGTH_EXCEEDED               =  25;
//* unsupported certificate purpose */
        INVALID_PURPOSE                    =  26;
//* certificate not trusted */
        CERT_UNTRUSTED                     =  27;
//* certificate rejected */
        CERT_REJECTED                      =  28;


        LDAPSSL_CERT_UTC_TIME  = 1;
        LDAPSSL_CERT_GEN_TIME  = 2;

        LDAPSSL_CERT_ACCEPT    = 0;
        LDAPSSL_CERT_REJECT    = -1;

type LDAPSSL_Private_Key = record
       length: nuint32;
       data  : pointer;
     end;
//{
//    unsigned long    length;
//    void            *data;
//} LDAPSSL_Private_Key, *pLDAPSSL_Private_Key;

    LDAPSSL_Cert = record
       length: nuint32;
       data  : pointer;
     end;
//{
//    unsigned long    length;
//    void            *data;
//} LDAPSSL_Cert, *pLDAPSSL_Cert;

    LDAPSSL_Cert_Validity_Period = record
      notBeforeTime: array[0..39] of char8;
      notBeforeType: int32;
      notAfterTime: array[0..39] of char8;
      notAfterType: int32;
    end;
//{
//   char  notBeforeTime[40];
//   int   notBeforeType;
//   char  notAfterTime[40];
//   int   notAfterType;
//} LDAPSSL_Cert_Validity_Period, *pLDAPSSL_Cert_Validity_Period;


type TcertVerifyFunc=function(pHandle: Pointer): int32; stdcall;


//* APIs */

function ldapssl_client_init(
           certFile: pchar8;
           reserved: pointer):ldapcode; stdcall;
//LDAP_F(int) ldapssl_client_init
//(
//   const char *certFile,
//   void *reserved
//);

function ldapssl_init(
           const certFile: pchar8;
           defport: int32;
           defsecure: int32):hLDAP; stdcall;
//LDAP_F(LDAP *) ldapssl_init
//(
//   const char *defhost,
//   int defport,
//   int defsecure
//);

function ldapssl_install_routines(
           var ld: hLDAP):ldapcode; stdcall;
//LDAP_F(int) ldapssl_install_routines
//(
//   LDAP *ld
//);

function ldapssl_client_deinit:ldapcode; stdcall;
//LDAP_F(int) ldapssl_client_deinit
//(
//   void
//);

function ldapssl_add_trusted_cert(
            cert: pointer;
           ltype: int32):ldapcode; stdcall;
//LDAP_F(int) ldapssl_add_trusted_cert
//(
//   void *cert,
//   int   type
//);

function ldapssl_set_verify_mode(
           mode: int32):ldapcode; stdcall;
//LDAP_F(int) ldapssl_set_verify_mode
//(
//   int   mode
//);

function ldapssl_get_verify_mode(
           var mode: int32):ldapcode; stdcall;
//LDAP_F(int) ldapssl_get_verify_mode
//(
//   int   *mode
//);

function ldapssl_set_verify_callback(
           certVerifyFunc: TcertVerifyFunc):ldapcode; stdcall;
//LDAP_F(int) ldapssl_set_verify_callback
//(
//   int (LIBCALL *certVerifyFunc)(void *)
//);

function ldapssl_get_cert_attribute(
           certHandle: pointer;
           attrID: int32;
           value : pointer;
           var length: int32):ldapcode; stdcall;
//LDAP_F(int) ldapssl_get_cert_attribute
//(
//   void          *certHandle,
//   int           attrID,
//   void          *value,
//   int           *length
//);

function ldapssl_get_cert(
           certHandle: pointer;
           ltype: int32;
           var cert: LDAPSSL_Cert):ldapcode; stdcall;
//LDAP_F(int) ldapssl_get_cert
//(
//   void          *certHandle,
//   int            type,
//   LDAPSSL_Cert  *cert
//);

function ldapssl_set_client_private_key(
           key: pointer;
           ltype: int32;
           password: pchar8):ldapcode; stdcall;
//LDAP_F(int) ldapssl_set_client_private_key
//(
//   void *key,
//   int   type,
//   void *password
//);

function ldapssl_set_client_cert(
           cert: pointer;
           ltype: int32;
           password: pchar8):ldapcode; stdcall;
//LDAP_F(int) ldapssl_set_client_cert
//(
//   void *cert,
//   int   type,
//   void *password
//);

function ldapssl_start_tls(
           var ls: hLDAP):ldapcode; stdcall;
//LDAP_F(int) ldapssl_start_tls
//(
//   LDAP *ld
//);

function ldapssl_stop_tls(
           var ls: hLDAP):ldapcode; stdcall;
//LDAP_F(int) ldapssl_stop_tls
//(
//   LDAP *ld
//);

implementation

function ldapssl_client_init;            external 'LDAPSSL.DLL';
function ldapssl_init;                   external 'LDAPSSL.DLL';
function ldapssl_install_routines;       external 'LDAPSSL.DLL';
function ldapssl_client_deinit;          external 'LDAPSSL.DLL';
function ldapssl_add_trusted_cert;       external 'LDAPSSL.DLL';
function ldapssl_set_verify_mode;        external 'LDAPSSL.DLL';
function ldapssl_get_verify_mode;        external 'LDAPSSL.DLL';
function ldapssl_set_verify_callback;    external 'LDAPSSL.DLL';
function ldapssl_get_cert_attribute;     external 'LDAPSSL.DLL';
function ldapssl_get_cert;               external 'LDAPSSL.DLL';
function ldapssl_set_client_private_key; external 'LDAPSSL.DLL';
function ldapssl_set_client_cert;        external 'LDAPSSL.DLL';
function ldapssl_start_tls;              external 'LDAPSSL.DLL';
function ldapssl_stop_tls;               external 'LDAPSSL.DLL';

end.
