(* $Novell: sslbind.c,v 1.16 2003/05/12 13:09:10 $ */
/**************************************************************************
*  Novell Software Developer Kit
*
*  Copyright (C) 2002-2003 Novell, Inc. All Rights Reserved.
*
*  THIS WORK IS SUBJECT TO U.S. AND INTERNATIONAL COPYRIGHT LAWS AND TREATIES.
*  USE AND REDISTRIBUTION OF THIS WORK IS SUBJECT TO THE LICENSE AGREEMENT
*  ACCOMPANYING THE SOFTWARE DEVELOPER KIT (SDK) THAT CONTAINS THIS WORK.
*  PURSUANT TO THE SDK LICENSE AGREEMENT, NOVELL HEREBY GRANTS TO DEVELOPER A
*  ROYALTY-FREE, NON-EXCLUSIVE LICENSE TO INCLUDE NOVELL'S SAMPLE CODE IN ITS
*  PRODUCT. NOVELL GRANTS DEVELOPER WORLDWIDE DISTRIBUTION RIGHTS TO MARKET,
*  DISTRIBUTE, OR SELL NOVELL'S SAMPLE CODE AS A COMPONENT OF DEVELOPER'S
*  PRODUCTS. NOVELL SHALL HAVE NO OBLIGATIONS TO DEVELOPER OR DEVELOPER'S
*  CUSTOMERS WITH RESPECT TO THIS CODE.
*
***************************************************************************
    sslbind.c
***************************************************************************
   Description: Demonstrates connecting to a to an ldap server using SSL
***************************************************************************)
program sslbind;
{$APPTYPE CONSOLE}

uses sysutils, ldap, ldap_ssl;

//static char usage[] =
//"\n Usage:   sslbind <host name> <port number> <login dn> <password>"
//"\n\t<cert file> <file type> \n"
//"  host name      = ldap server name or IP address\n"
//"  port number    = port to use - 636 is ldap ssl default\n"
//"  login dn       = user name to login as\n"
//"  password       = user password\n"
//"  cert file      = trusted root certificate file\n"
//"  file type      = DER=der encoded file, B64=b64 encoded file\n"
//"\n Example: sslbind Acme.com 636 cn=admin,o=Acme secret myKey.der DER\n";


{$IFDEF VER210}
var timeout : timeval = (tv_sec:10;tv_usec:0); //* 10 second connection timeout */
{$ELSE}
const timeout : timeval = (tv_sec:10;tv_usec:0); //* 10 second connection timeout */
{$ENDIF}
var version, rc, ldapPort: int32;
    ldapHost  : AnsiString;
    loginDN   : AnsiString;
    password  : AnsiString;
    certFile  : AnsiString;
    fileType  : AnsiString;
    fileEncoding: Integer;

    ld        : hLDAP;
    resultBuf : LDAPMessage;
    entry     : LDAPMessage;
    attrs     : array[0..1] of pchar8;
    dn        : pchar8;

begin
    if (paramcount()<>6) then begin
      writeln('Usage:   sslbind <host name> <port number> <login dn> <password> <cert file> <file type>');
      writeln('  host name      = ldap server name or IP address');
      writeln('  port number    = port to use - 636 is ldap ssl default');
      writeln('  login dn       = user name to login as');
      writeln('  password       = user password');
      writeln('  cert file      = trusted root certificate file');
      writeln('  file type      = DER=der encoded file, B64=b64 encoded file');
      writeln('    Example: sslbind Acme.com 636 cn=admin,o=Acme secret myKey.der DER');
      halt(1);
    end;

    ldapHost     := paramstr(1);
    ldapPort     := StrToInt(paramstr(2));
    loginDN      := paramstr(3);
    password     := paramstr(4);
    certFile     := paramstr(5);
    fileType     := paramstr(6);


    //* Set LDAP version to 3 and set connection timeout. */
    version := LDAP_VERSION3;
    writeln('ldap_set_option');
    ldap_set_option(NIL, LDAP_OPT_PROTOCOL_VERSION, version);
    ldap_set_option(NIL, LDAP_OPT_NETWORK_TIMEOUT, timeOut);

    //*
    //* Validate the fileType argument
    //*/
    if (Pos('DER',fileType)<>0) OR (Pos('der',fileType)<>0) then begin
       fileEncoding := LDAPSSL_CERT_FILETYPE_DER;
    end else begin
      if (Pos('B64',fileType)<>0) OR (Pos('b64',fileType)<>0) then begin
        fileEncoding := LDAPSSL_CERT_FILETYPE_B64;
      end else begin
        writeln('Invalid certificate file type');
        fileEncoding := 0;
        halt(1);
      end;
    end;


    ///*
    // * Initialize the ssl library.  The first parameter of
    // * ldapssl_client_init, if specified, must be a DER-encoded
    // * certificate file.  If NULL is given for this parameter, the
    // * certificate file may be specified by the ldapssl_add_trusted_cert
    // * API, which accepts either DER or B64-encoded files.
    // *
    // * ldapssl_client_init is an application level initialization not a
    // * thread level initilization and should be done once.
    // */

    writeln('ldapssl_client_init');
    rc := ldapssl_client_init(nil,         //* DER encoded cert file  */
                              nil);        //* reserved, use NULL */

    if (rc <> LDAP_SUCCESS) then begin
      writeln('ldapssl_client_init error: ', rc);
      halt(1);
    end;


    ///*
    // * Add the trusted root certificate file.  Multiple trusted root
    // * certificates can be added with ldapssl_add_trusted_cert.  Applications
    // * can use this functionality to specify groups of trusted servers.
    // */
    writeln('ldapssl_add_trusted_cert');
    rc := ldapssl_add_trusted_cert(pchar8(certFile), fileEncoding);
    if (rc <> LDAP_SUCCESS) then begin
      writeln('ldapssl_add_trusted_cert error: ', rc);
      ldapssl_client_deinit();
      halt(1);
    end;


    ///*
    // * create a LDAP session handle that is enabled for ssl connection
    // */
    writeln('ldapssl_init');
    ld := ldapssl_init(pchar8(ldapHost),//* host name */
                       ldapPort,        //* port number */
                       1);              //* 0- clear text, 1-enable for ssl */

    if (ld = nil) then begin
      writeln('ldapssl_init error' );
      ldapssl_client_deinit();
      halt(1);
    end;

    writeln('ldap_simple_bind_s');
    rc := ldap_simple_bind_s(ld, pchar8(loginDN), pchar8(password));

    if (rc <> LDAP_SUCCESS) then begin
      writeln('ldap_simple_bind_s error: ', rc, ' ',ldap_err2string( rc ));
      ldap_unbind_s(ld);
      ldapssl_client_deinit();
      halt(1);
    end;

    writeln('SSL bind successful - performing search');


    ///*
    // * do a simple search just to verify things worked
    // */
    attrs[0] := LDAP_NO_ATTRS;
    attrs[1] := nil;

    writeln('ldap_search_ext_s');
    rc := ldap_search_ext_s(ld, '' , LDAP_SCOPE_ONELEVEL, '(objectClass=*)',
                           addr(attrs), 0, nil, nil, nil,
                           LDAP_NO_LIMIT, resultBuf);

    if (rc = LDAP_SUCCESS) then begin
       entry := ldap_first_entry(ld, resultBuf);

       while (entry <> nil) do begin
          dn := ldap_get_dn(ld, entry);
          writeln('Object: ', dn);

          ldap_memfree(dn);

          entry := ldap_next_entry(ld, entry);
       end;

       ldap_msgfree(resultBuf);
    end else begin
       writeln('ldap_search_s error: ', rc, ' ', ldap_err2string(rc));
    end;

    ldap_unbind_s(ld);

    ///*
    // * Uninitialize the LDAP ssl library
    // */
    ldapssl_client_deinit();
END.
